@@ -1,6 +1,6 @@
 package forks::BerkeleyDB::shared::array;
 
-$VERSION = 0.02;
+$VERSION = 0.03;
 use strict;
 use warnings;
 use BerkeleyDB 0.27;
@@ -253,20 +253,16 @@ sub SPLICE {
 #---------------------------------------------------------------------------
 sub UNTIE {
 	my $self = shift;
-	my $status = eval { $self->db_sync(); };
-	$status = eval { $self->db_close(); };
-	return defined $status && $status == 0 ? 0 : 1;
+	eval { $self->db_sync(); };
 }
 
 sub DESTROY {
 	my $self = shift;
-	my $status = eval { $self->db_sync(); };
-	$status = eval { $self->db_close(); };
-	return defined $status && $status == 0 ? 0 : 1;
+#	eval { $self->db_sync(); };
+	$self->SUPER::DESTROY(@_) if $self;
 }
 
 #---------------------------------------------------------------------------
-
 1;
 
 __END__
